#ifndef MONET_MAP_DRAWING_H
#define MONET_MAP_DRAWING_H

/***************************************************************************/
/***************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/Drawing.h,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/***************************************************************************/
/***************************************************************************/

#include <monet/db/Db.h>
#include <monet/msh/OrdSet.h>
#include <monet/draw/Shape.h>

/***************************************************************************/
/*  List of drawings                                                       */
/***************************************************************************/

typedef MonetOrdSet DrawingList;
#define extract_drawingList(list,o)	extract_MonetOrdSet (list, (caddr_t) o)
#define Drawing_createList()		(DrawingList) create_MonetOrdSet ()
#define reset_drawingList(list)		reset_MonetOrdSet (list)
#define Drawing_freeList(list)		free_MonetOrdSet (list)
#define insert_drawingList(list, o)	insert_MonetOrdSet(list, (caddr_t) o)
#define isEmpty_drawingList(list)	isEmpty_MonetOrdSet(list)
#define getN_drawingList(list, n)	(Drawing) getN_MonetOrdSet(list,n)
#define first_drawingList(list)		(Drawing) first_MonetOrdSet(list)
#define next_drawingList(list)		(Drawing) next_MonetOrdSet(list)
#define sizeOf_drawingList(list) 	sizeOf_MonetOrdSet(list)
#define second_drawingList(list)	(Drawing) second_MonetOrdSet(list)
#define isMember_drawingList(list, o)	isMember_MonetOrdSet(list, (caddr_t) o)

#define FOR_DRAWINGLIST(o, list) FOR_ORDSET(o, list, Drawing)


/***************************************************************************/
/*  Others */
/***************************************************************************/

#include <monet/draw/Draw.h>
#include <monet/map/Label.h>
#include <monet/map/Draw.h>
#include <monet/map/Map.h>
#include <monet/map/Lines.h>

/***************************************************************************/
/*  A rectangle with a label                                               */
/***************************************************************************/

typedef struct monetBox {
  Position  x,y;
  Dimension width, height;
  String label;
} MonetBox;

#define COPYMONETBOX(to, from) { to.x = from.x; to.y =from.y; \
			to.width = from.width; to.height = from.height; \
			to.label = from.label; }
#define NULL_MonetBox {0,0,0,0,NULL}

/***************************************************************************/
/*** Cannot be selected as a supercomponent and normally at the same time */
/***************************************************************************/

typedef enum {
	select_none,
	select_first,
	select_super,
	select_normal
	}
	SelectState;


/***************************************************************************/
/*  The drawing structure                                                  */
/*  Each Drawing.has exactly one entity                                    */
/***************************************************************************/

struct drawing {

  /***************************************************************************/
  /*  Each drawing is an element of a map, a map can be a drawing to         */
  /***************************************************************************/

  Map map;     	/*  Pointer to the Map object that this is displayed on */
#define Drawing_mapOf(d)	d->map

  /***************************************************************************/
  /*  If this has children  */
  /***************************************************************************/

  Map submap;
  MonetBool submap_visible;
  DrawingList map_interfaces;

  /***************************************************************************/
  /*  A couple of general purpose selection lists are provided               */
  /***************************************************************************/

  SelectState	select;
#define SELECT(o)	(o->select)

  /***************************************************************************/
  /* Has the system geometry been changed from its default */
  /***************************************************************************/

  MonetBool		resized;

  /***************************************************************************/
  /*  If it's neighbours change or is moved then redraw the connectors.      */
  /***************************************************************************/

  MonetBool redraw_connectors;

  /***************************************************************************/
  /*  Position on the map, !!! Should be a monet box                         */
  /***************************************************************************/
  
  Position	y,x;
  Dimension	stretch_width, stretch_height;
  char *	label;  /*  The label that is actually displayed */
  int           labellen;
  char *        given_label;  /* Label given by user */
  MonetBox	list;  /* Position when listed */

  /***************************************************************************/
  /*  State attributes                                                       */
  /*  Define exactly what the drawing looks like                             */
  /*  These values are derived mainly from the state of the entity           */
  /***************************************************************************/

  String	label2;		/* Bottom right corner */
  MonetShapeState  state, true_state;
  MonetShape shape;
#define Drawing_shapeOf(d) ((d)->shape)
  MonetLabelState  primary_label, secondary_label;

  /***********************************************************************/
  /*  For interfaces only  */
  /***********************************************************************/

  MonetConnectingLine from, to;
#define DRAWING_FROM(d) ((d)->from->system)
#define DRAWING_TO(d) ((d)->to->system)

  /***************************************************************************/
  /*  List of drawings attached to an entity                                 */
  /*  Each drawing must have exactly one entity                              */
  /***************************************************************************/

  Drawing		next;
  MonetEntity		entity;
#define ENTITY(d) ((d)->entity)
#define Drawing_classOf(d)  MonetEntity_classOf((d)->entity)

/*  Obsolete */

/***************************************************************************/
};	/* End struct drawing */
/***************************************************************************/

/***************************************************************************/
/*  MACROS                                                                 */
/***************************************************************************/

#define DRAWING(o) (o)
#define LISTBOX(o) (o->list)
/* These should not be directly assigned to, routines are provided for that */
#define X(o)	(o->x)
#define Y(o)	(o->y)
#define SHOW(o)		(o->shown)
#define WIDTH(o)	(o->stretch_width)
#define HEIGHT(o)	(o->stretch_height)
#define REVERSEVIDEO(o)	(o->reverse_video)
#define LABEL(o)	(o->label)
#define LABEL2(o)	(o->label2)

#define Drawing_title(d)	MonetEntity_title (ENTITY(d))
#define Drawing_isInterface(d)  MonetEntity_isInterface(ENTITY(d))
#define Drawing_isClass(d)      MonetEntity_isClass(ENTITY(d))

/***************************************************************************/
/*  Function Prototypes                                                    */
/***************************************************************************/

extern	Drawing	Drawing_create
  PR((String, MonetEntity, Map, Position, Position, Dimension, Dimension));
extern	Drawing	Drawing_create_by_name
  PR((String, Map, Position, Position, Dimension, Dimension));
extern Drawing Drawing_create_from_entity
  PR((MonetEntity, Map, MonetShape, char *,
      Position, Position, Dimension, Dimension));
extern	MonetBool	Drawing_isMap
  PR((Drawing));
extern	void	Drawing_free
  PR((Drawing));
extern MonetBool Drawing_isResizable
  PR((Drawing));
extern	void	Drawing_unset_resized
  PR((Drawing));
extern	void	Drawing_set_resized
  PR((Drawing));
extern	MonetBool	Drawing_prepare
  PR((Drawing));
extern void Drawing_change_trigger
  PR((MonetEntity));
extern	MonetBool	Drawing_position
  PR((Drawing, Position, Position));
extern	MonetBool	Drawing_dimension
  PR((Drawing, Position, Position, Dimension, Dimension));
extern	void	Drawing_reset_list_label
  PR((Drawing));
extern	MonetBool	Drawing_isMoveable
  PR((Drawing));
extern	Drawing	Drawing_from_interface
  PR((Drawing));
extern	Drawing	Drawing_to_interface
  PR((Drawing));
extern	MonetBool Drawing_in_box
  PR((Drawing, Position, Position, Dimension, Dimension));
extern	MonetBool	Drawing_is_marked
  PR((Drawing, Position, Position));
extern	MonetBool Drawing_are_linked
  PR((Drawing, Drawing));
extern	void	MonetEntity_select_drawings
  PR((MonetEntity));
extern	Drawing	get_drawing
  PR((MonetEntity, Map));
extern Drawing get_submap_drawing
  PR((MonetEntity, Map));
extern	Drawing	Drawing_find
  PR((String, MonetEntity, Map));
extern	Drawing Drawing_get_current
  PR((void));
extern void Drawing_set_current
  PR((Drawing new));

/***************************************************************/
/***  For marking systems selected by the user ***/
/*    The poor naming is historical !!! */
/***************************************************************/

/*  Return the most recently selection drawing in the list */
extern Drawing	current_Selection
  PR((Map));
extern unsigned	numberOf_Selection
  PR((Map));
extern void free_selection
  PR((Map));
extern Drawing	next_selection
  PR((Map));
extern Drawing	second_selection
  PR((Map));
extern void	add_selection
  PR((Drawing));
extern void	remove_selection
  PR((Drawing));
extern void	toggle_selection
  PR((Drawing));
extern void select_no_draw
  PR((Drawing));
extern	MonetBool	Drawing_isSelected
  PR((Drawing));

#define FOR_SELECTIONS(map, o)	\
  for (o=current_Selection (map); o != NULL; o=next_selection (map))
#define FOR_SELECTED_ENTITY(map, o)	{ \
		Drawing for_d; \
		FOR_SELECTIONS(map, for_d) { \
			o = ENTITY(for_d);
#define END_SELECTED_ENTITY	}}

extern Drawing current_class
     PR((void));
extern void mark_class
  PR((Drawing o));
extern void unmark_class
  PR((void));

extern MonetBool Drawing_isMinimal
  PR((Drawing));

extern MonetBool Drawing_isElipse
  PR((Drawing));

#endif

/***************************************************************************/
/***************************************************************************/
/*  $Header: //gaia/group.local/computer/monet/lib/map/RCS/Drawing.h,v 1.1 1991/06/11 11:26:10 a38 Exp a38 $  */
/***************************************************************************/
/***************************************************************************/

