#ifndef MONET_MAP_H
#define MONET_MAP_H

#include <monet/draw/Draw.h>
#include <monet/map/Backup.h>

/***********************************************************************/
/*  Layout of elements on map                                          */
/***********************************************************************/

typedef enum {
  drawnas_map = 0,
  drawnas_list = 1,
  drawnas_graph = 2 }
  DrawnAs;

/***********************************************************************/
/*  How network broders are drawn  */
/* Networks drawn as elipses/circles */
/***********************************************************************/

typedef enum {
  netb_box = 0,
  netb_round = 1,
  netb_minimal = 2 }
  MonetNetBorder;
#define MonetNetBorderDefault (netb_box)

/***********************************************************************/
/*  Map datastructure */
/*  Basically this is just a widget for displaying a map on.  When a   */
/*  map is changed this is reflected on the map areas.                 */
/*  This could just as well be implemented as a new widget class       */
/*  !!! This should all be private                                     */
/***********************************************************************/

struct map {

  /*********************************************************************/
  /*  Backup for this map, deletions and changes saved here  */
  /*********************************************************************/

  Backup backup;

  /*********************************************************************/
  /*  For Positioning new elements, !!!                                */
  /*********************************************************************/

  Position unplaced_x, unplaced_y;

  /*********************************************************************/
  /*  Have the subcomponents been saved to disc yet                    */
  /*********************************************************************/

  MonetBool saved;
#define Map_is_saved(map)  (map->saved || map->entity == NULL)

  /*********************************************************************/
  /*  Drawing enabled/disabled, changable or not  */
  /*********************************************************************/

  MonetBool drawing_allowed, can_change;
#define isDrawingAllowed(map)  ((map) && (map)->drawing_allowed)

  /*********************************************************************/
  /*  -- A map is basically a list of drawings  */
  /*********************************************************************/

  DrawingList elements;
#define	Map_elements(map)  ((map)->elements)
  DrawingList selected;  /*  selcted elements  */

  MonetBool class_map;  /* Is this map only for drawing classes ? */
#define Map_classesOnly(map) ((map)->class_map)
#define Map_set_classesOnly(map) (map)->class_map = True;


  /*********************************************************************/
  /*  The name of the map file  */
  /*********************************************************************/

  String filename;
  MonetEntity entity;
#define Map_is_loaded(map)	((map)->entity)

  /*********************************************************************/
  /*  Widgets associated with this map area                            */
  /*********************************************************************/

  /*  Widget popup;  Not wanted !!! */
  /* Widget mapMenu; !!! */
#define map_menu(s) MAPAREA(s)->mapMenu
#define map_popup(map)    (MAPAREA(map)->popup)

  Widget mapWidget;
  Region clip_region;		/* For graphics exposure */
#define Map_clip_region(map) (map->clip_region)

  /*********************************************************************/
  /*  How the map is actually drawn                                    */
  /*********************************************************************/

  ConnectorState connectors;
  unsigned grid_x, grid_y;
  DrawnAs drawnas;		/* Draw a map, or just a list of systems  */
#define Map_drawnas(o)  o->drawnas

  MonetReportFormat  report_format;
  String default_domain;   /* -- hosts in this domain will have short labels */
  MonetShape background_shape;  /* -- Graphics for background */
  Dimension shape_width, shape_height;
  String background_colour;
  unsigned	default_domain_len;
  MonetBool draw_shape;
  MonetBool marked;	/*  Has this map been marked  */
  MonetNetBorder  net_border;
  MonetAttribute
    primary_label, secondary_label,
    scale_from_line, scale_to_line;
  int scale_from_by, scale_to_by; /* Denominator for line scaling */

  /*********************************************************************/
  /* The position and dimensions of the maps X Window on the screen    */
  /*********************************************************************/

  float x_scale, y_scale, scale_by;
  Position x_centre, y_centre;

  /*********************************************************************/
  /*  Actual dimensions of the map                                     */
  /*********************************************************************/

  Dimension xl,yl,xr,yr;
#define CLEAR_DIMENSIONS(map) map->xl = map->yl = map->xr = map->yr = 0;
#define ARESET_DIMENSIONS(map)   (map->xr && map->yr)

  /*********************************************************************/
  /*  The maps a linked together in a list  */
  /*********************************************************************/

  Map next;
};

#define FORALLMAPS(m) for (m = Map_first (); m != NULL ; m = Map_next (m))

#define NULMAP ((Map) 0)

/***********************************************************************/
/*  Initialise, supply routines for handling when map needs redrawing
    and for beginning and ending redrawings of a single drawing,
    these may be null */
/***********************************************************************/

typedef void (* MapChangedCallback)
     PR((Map, Dimension width, Dimension height, MonetBool redraw));

typedef void (* MapFreeCallback)
     PR((Map));

typedef void (* MapCurrentCallback)
     PR((Map previous, Map, MonetBool swap));

typedef MonetBool (* DrawingBeginCallback)
     PR((Map));

typedef void (* DrawingEndCallback)
     PR((Map));

extern void	MonetMap_init
  PR((MapChangedCallback, DrawingBeginCallback, DrawingEndCallback,
      MapCurrentCallback, MapFreeCallback));

/***********************************************************************/
/*  Current map  */
/***********************************************************************/

extern Map MapCurrent_get
  PR((void));

extern void MapCurrent_set
  PR((Map new, MonetBool swap));

extern void  MapCurrent_swap
  PR((void));
extern void  MapCurrent_set_next
  PR((void));

/***********************************************************************/
/*  Do an action for a set of selected drawings  */
/***********************************************************************/

typedef void (* DoDrawingCallback) PR((Drawing));

typedef MonetBool (* DoTestRedrawCallback) PR((Drawing));

extern void	Map_parse_select
  PR((int argc, String *arg, DoDrawingCallback));

extern void	Map_parse_select_redraw
  PR((int argc, String *arg, DoTestRedrawCallback, DoDrawingCallback));

extern void	Map_parse_select_moved_redraw
  PR((int argc, String *arg, DoTestRedrawCallback, DoDrawingCallback));

#define NULTEST ((DoTestRedrawCallback) 0)

/***********************************************************************/

/***********************************************************************/

extern void Scale2ActualSize
  PR((float x_scale, float y_scale,
      Dimension, Dimension, Dimension*, Dimension*));

extern MonetBool Map_reset_corners
  PR((Map, MonetBool force_check));

extern void Map_set_resized PR((void));

extern void	MonetMap_finish
  PR((void));
extern void	MonetMap_redraw_all
  PR((void));
extern void	MonetMap_changed
  PR((void));

extern Map Map_create_class
  PR((void));
extern Map	Map_create
  PR((MonetEntity));
extern void	Map_free
  PR((Map map));
extern Map	Map_discard
  PR((Map map));

extern Map	Map_first
  PR((void));
extern Map	Map_next
  PR((Map current));

extern Widget	Map_widget
  PR((Map));
extern Map	Map_root
  PR((Map));
extern Map	Map_get_map_for_entity
  PR((MonetEntity));
extern void	Map_set_domain
  PR((Map, String));
extern String	Map_get_domain
  PR((Map));

extern void	Map_get_dimensions
  PR((Map, Position *, Position *, Dimension *, Dimension *));

extern void	Map_set_background_shape
  PR((Map, MonetShape));
extern MonetShape	Map_get_background_shape
  PR((Map));
extern void	Map_set_background_colour
  PR((Map, String));
extern String	Map_get_background_colour
  PR((Map));
extern void	Map_request_redraw
  PR((Map));
extern void Map_request_changed_redraw
  PR((Map));
extern void Drawing_changed_redraw
  PR((Drawing, MonetBool moved));

extern MonetBool	Map_corners
  PR((Map, Position*, Position*, Position*, Position*, MonetBool force_check));

extern void	Map_adjust_corners
  PR((Map, Drawing, MonetBool adding));

extern void	Map_set_grid
  PR((Map, unsigned x, unsigned y));

extern MonetBool	Map_insert_drawing
  PR((Map, Drawing, MonetShape, char *,
      Position, Position, Dimension, Dimension));

extern Drawing	Map_insert_drawing_class
  PR((Map, MonetEntity, Position, Position));

extern void	Map_extract_drawing		PR((Map, Drawing));
extern void	Map_extract_all_drawings	PR((Map));
extern MonetBool	Map_can_insert_drawing		PR((Map, MonetEntity));
extern String	Map_title			PR((Map));

extern MonetBool	Map_load			PR((Map, String filename));
extern MonetBool	Map_write			PR((Map, String filename));
extern Map	Map_make			PR((Map, String name));
extern Map	Map_make_class_map		PR((Map map));

extern MonetBool	Map_ListRemoteDirectory		PR((String host, int port));

extern String	drawnas2string			PR((Drawnas));
extern DrawnAs	string2drawnas			PR((String));

extern void	Map_set_changed			PR((Map));
extern void	Map_disable_set_changed		PR((Map));
extern void	Map_enable_set_changed		PR((Map));

extern Drawing Map_get_marked
  PR((Map *map,
      Position *x_map, Position *y_map, Position *x_win, Position *y_win,
      Dimension *w_win, Dimension *h_win));

extern Drawing Map_actual2map
  PR((Map *map, 
      Position *x_map, Position *y_map,
      Dimension *w_map, Dimension *h_map));

extern MonetBool Map_select_in_box
  PR((Map map,
      Position xloc, Position yloc,
      Dimension width, Dimension height));

extern void Map_adjust_for_grid
  PR((Map,
      Position x, Position y,
      Position *newx, Position *newy));

extern MonetBool		explore_connectedSystems
  PR((Drawing));

extern MonetBool		rename_entity
  PR((MonetEntity, String));

extern Map MonetString2Map
  PR((char *));

extern char * Map_nice_name
  PR((Map));

extern MonetNetBorder String2MonetNetBorder
  PR((char *));

extern char * MonetNetBorder2String
  PR((MonetNetBorder net));

#endif
