#ifndef  MONET_ENTITY.H
#define  MONET_ENTITY.H

/***************************************************************************/
/***************************************************************************/
/*  $Header: MonetEntity.h,v 1.1 91/10/01 16:06:58 a38 Exp $ */
/***************************************************************************/
/***************************************************************************/

#include <monet/msh/General.h>
#include <monet/msh/EntityList.h>


/***************************************************************************/
/*  Forward definition of some private datastructures                      */
/***************************************************************************/

typedef struct monet_entity	   * MonetEntity;
typedef struct map		   * Map;
typedef struct drawing		   * Drawing ;
typedef struct class_data	   * MonetEntityClassData;
typedef struct monet_request	   * MonetRequest;
typedef struct monet_attribute     * MonetAttribute;
typedef struct monet_type          * MonetType;
typedef struct monet_report_format * MonetReportFormat;
typedef struct monet_event	   * MonetEvent;
typedef struct monet_redirect * MonetRedirect;

#include <monet/msh/Script.h>
#include <monet/msh/EntityStack.h>

/***************************************************************************/
/*  Constants                                                              */
/***************************************************************************/

#define CONTINUATION_CHAR '$'
#define COMMENT_CHAR '#'
#define MonitorEndCode '\n'
#define MonitorSepCode '\n'

#define MAXCONNECTIONS		-1   /* !!! */
#define ENTITY_MAXPATHLEN   1024
#define ENTITY_MAXNAMELEN   64
#define MONET_MAX_ATTRIBUTES 64

#define NULENT ((MonetEntity) NULL)

/***************************************************************************/
/*  Each entity has two sets of status values.  One set has the latest     */
/*  values , the other has the previous.  */
/*  A status value is either a counter or a 32 bit value. */
/*  Each entity has an array of Status Values.  */
/***************************************************************************/

typedef union monet_value {
    caddr_t pointer;
    unsigned integer;
    MonetBool boolean;
    char character;
    String * string_pointer;
    MonetBool * boolean_pointer;
    long * long_pointer;
    MonetStr str;
} MonetValue;

typedef MonetValue MonetStatusSet [MONET_MAX_ATTRIBUTES];

/***************************************************************************/
/*  Default attributes  */
/***************************************************************************/

#define MonetAttribute_TITLE_NUM      1
#define MonetAttribute_CLASS_NUM      2

#define MONETGETINTEGER(entity, id)  \
  ((entity)->status[id-1].integer)
#define MONETGETBOOLEAN(entity, id)  \
  ((entity)->status[id-1].boolean)
#define MONETGETPOINTER(entity, id)  \
  ((entity)->status[id-1].pointer)
#define MONETGETMONETSTR(entity, id)  \
  ((entity)->status[id-1].str)

#define FORMAT(c)      (CLASS(c)->title_format)

/***************************************************************************/
/*  Mask to see which attributes are changed or set  */
/***************************************************************************/

#define MonetBITS (sizeof (long) * 8)       /* bits per mask */
#define MonetWORDS (howmany(MONET_MAX_ATTRIBUTES, MonetBITS))
typedef struct monet_mask_set {
        long bits[MonetWORDS];
} MonetMask;

/***************************************************************************/
/*  A protocol entity MonetEntity                                          */
/***************************************************************************/

struct monet_entity {
  /***************************************************************************/
  /*  Definition of a visual object.                                         */
  /***************************************************************************/

#define MONETTITLE(entity) \
  MONETGETMONETSTR((entity),MonetAttribute_TITLE_NUM)

#define TITLE(entity) \
    (MonetStr2String (MONETTITLE(entity)))

#define CLASSOFENTITY(entity) \
	MONETGETPOINTER((entity),MonetAttribute_CLASS_NUM)
#define MonetEntity_classOf(o) \
	  ((o) == NULL ? NULL : (MonetEntity) CLASSOFENTITY(o))

  /*************************************************************************/
  /*  A list of functions for this entity  */
  /*************************************************************************/

  MonetMember members;

  /***************************************************************************/
  /*  Objects referencing the entity.  Currently two types, each is          */
  /*  (or should be) a list of objects all referencing this particular       */
  /*  entity.  !!! Should generalise this so that no changes need be made    */
  /*  here for new types.                                                    */
  /***************************************************************************/
  
  MonetRequest       requests;	/* List of requests to this entity */
  Map		map;
  Drawing       drawing;	/* List of visual objects for this entity */

  /***************************************************************************/
  /*  Special stuff for entities that are classes                            */
  /***************************************************************************/
  
  MonetEntityClassData	class_stuff;	/*  For Classes Only */
#define MonetEntity_isClass(o)	(o->class_stuff != NULL)

  /***************************************************************************/
  /*  The following defines the entities status                              */
  /*  The entity database is described as a stack of networks.  Each entity  */
  /*  is either a node/system or a link/interface.  A node is connected to   */
  /*  other nodes via a set of interfaces, while an interface connects at    */
  /*  at most two systems together.                                          */
  /***************************************************************************/

  MonetEntityList	interfaces;	/* For Systems Only */
#define MonetEntity_interfaces(o)		((o)->interfaces)

  MonetEntityList	supercomponents;
#define MonetEntity_supercomponents(o)	(o->supercomponents)

  MonetEntityList	subcomponents;
#define MonetEntity_subcomponents(o)	(o->subcomponents)

  MonetStatusSet status;
  unsigned size_status_set; /*  Number of elements in the set  */
  MonetMask mask;
#define MASK(o)  ((o)->mask)

  /***************************************************************************/
  /*  Where to send query requests to                                        */
  /***************************************************************************/

  MonetRedirect	redirect;
  MonetEvent event;
  MonetStackEntry stacks;
  MonetEntity next;

/***************************************************************************/
};  /*****  End of struct monet_entity ****/
/***************************************************************************/

extern MonetReportFormat Monet_log_format, Monet_report_format;

/***************************************************************************/
/*  Attribute access  */
/***************************************************************************/

/***************************************************************/
/*  A list of all the entities  */
/***************************************************************/

extern MonetEntity Entities;
extern MonetAttribute
  MonetClassAttribute; /* The class attribute */

/***************************************************************/
/*  Attribute routines  */
/*
  Currently there is one routine for accessing data,
  might be nicer to have one routine per operation.

typedef struct monet_access MonetAccess;
struct monet_access {
  void (* init) PR(());
  void (* free) PR(());
  void (* get) PR(());
  void (* get_shallow) PR(());
  void (* set) PR(());
};
*/
/***************************************************************/

typedef enum {
	attribute_unset,
	attribute_set,
	attribute_get,		/*  Use inheritance  */
	attribute_get_shallow,	/*  Do not use inheritance  */
	attribute_init,		/*  Do not call changed trigger */
	attribute_free
	} AttributeOperation;

typedef enum {
	attribute_ascii,
	attribute_compact_ascii,
	attribute_binary,	/*  One day  */
	attribute_asn1		/*  For sake of completeness  */
	} AttributeFormat;

typedef String  (* AttributeRoutine)
     PR((MonetEntity, unsigned id, AttributeOperation, AttributeFormat,
	 MonetBool null_if_default, String));

/***************************************************************/
/*  !!! I hate this, the type of the attribute about to be     */
/*  !!! converted.
/***************************************************************/

extern caddr_t Monet_type_being_accessed_data ();

/***************************************************************************/
/*  Mask to see which attributes are changed or set  */
/***************************************************************************/

#define MonetMask_SET(n, p) \
    ((p)->bits[(n)/MonetBITS] |= (1 << ((n) % MonetBITS)))
#define MonetMask_CLR(n, p)\
  ((p)->bits[(n)/MonetBITS] &= ~(1 << ((n) % MonetBITS)))
#define MonetMask_ISSET(n, p)\
  ((p)->bits[(n)/MonetBITS] & (1 << ((n) % MonetBITS)))
#define MonetMask_MERGE(p,q)\
{ int i; for (i=0; i<MonetWORDS; ++i)  ((p)->bits[i] |= (q)->bits[i]); }

/***************************************************************/
/*  generic function for accessing attributes  */
/***************************************************************/

#define ACCESS(routine_name, ACCESS_MEMBER, TOSTRING, FROMSTRING, TOCOMPACT, FROMCOMPACT, TYPE, NEQ, SET, UNSET) \
static String routine_name (o, id, op, format, null_default, data) \
       MonetEntity o; \
       unsigned id; \
       AttributeOperation op; \
       AttributeFormat format; \
       MonetBool null_default; \
       String data; \
{ \
    TYPE value, *place;                          \
                                              \
    if ( ! o)                          \
		return NULL;                   \
    place = &(ACCESS_MEMBER(o));   \
    if (op == attribute_init) {             \
		  SET(*place, UNSET);             \
	} else if (op == attribute_unset || op == attribute_free) {     \
                if (NEQ (*place, UNSET)) { \
		  SET(ACCESS_MEMBER(o), UNSET);             \
                  MonetMask_SET(id-1, &MASK(o)); \
		}   \
	} else if (op == attribute_set) {           \
		if (format == attribute_compact_ascii) \
			if (data)                                    \
				value = (TYPE) (FROMCOMPACT(data)); \
			else                    \
				value = UNSET; \
		else \
			value = (TYPE) FROMSTRING(data);            \
                if (NEQ (*place, value)) { \
		  SET(*place, value);             \
		  return ""; \
		  }   \
		  return NULL; \
       	    	} else if (op == attribute_get_shallow) {     \
	} else if (op == attribute_get) {                 \
	       while (*place == UNSET) {             \
		  if (o = MonetEntity_classOf (o))                \
                     place = &(ACCESS_MEMBER(o));   \
                  else \
                     break; \
	       } \
	} else {                                                  \
          MonetSysError ("Unknown attribute access operation [%d]", (int) op); \
	}                                                    \
	if (null_default && *place == UNSET)		           \
		return NULL;                                \
	if (format == attribute_compact_ascii)               \
		return TOCOMPACT(*place);                     \
	return TOSTRING(*place);                     \
}

/***************************************************************************/
/*  Function Prototypes                                                    */
/***************************************************************************/

typedef void (* MonetStatusTrigger) PR((MonetEntity));

extern MonetEntityList MonetEntity_instances
  PR((MonetEntity class));
extern MonetEntityList MonetEntity_subclasses
  PR((MonetEntity class));

extern MonetEntity MonetEntityTopClass
  PR((void));

extern MonetBool	MonetEntity_save_db
  PR((String filename));
extern MonetBool	MonetEntity_load_db
  PR((String filename));
extern MonetEntity	MonetEntity_create_instance
  PR((MonetEntity class, String title));

extern MonetEntity	MonetEntity_create_class
  PR((MonetEntity class, String title));
extern void	MonetEntity_free
  PR((MonetEntity));
extern String	MonetEntity_title
  PR((MonetEntity));
extern MonetBool	MonetEntity_subclass
  PR((MonetEntity of, MonetEntity sub));

extern MonetBool	MonetEntity_rename
  PR((MonetEntity, String));
extern MonetBool	MonetEntity_load_line
  PR((MonetEntity *, String line));
extern MonetBool  MonetEntity_any_map
  PR((MonetEntity));
extern MonetEntity	String2MonetEntityClass
  PR((String));
extern MonetEntity	String2MonetEntity
  PR((String));

extern MonetEntity	parse_title
  PR((String, String*, MonetEntity *, MonetBool *));

extern MonetBool	MonetEntity_decodeCompactAscii
  PR((MonetEntity, String *));
extern String	MonetEntity2Report
  PR((MonetEntity, String header,
      MonetBool all,
      MonetBool compact,
      MonetBool null_if_empty));

extern void MonetEntity_class_instance_parts
  PR((char* string, char **instance, char**class));

/***************************************************************************/
/*  Attributes  */
/***************************************************************************/

extern void MonetStatusSet_create
  PR((MonetEntity));
extern void MonetStatusSet_free
  PR((MonetEntity));
extern MonetType MonetAttribute2MonetType
  PR((MonetAttribute));
extern unsigned MonetAttribute2AttributeNumber
  PR((MonetAttribute));
extern void	MonetEntity_set_attribute
  PR((MonetEntity, MonetAttribute, String));
extern void	MonetEntity_set_attributes
  PR((MonetEntity,MonetAttribute*,String*));
extern String	MonetEntity_get_shallow_attribute
  PR((MonetEntity, MonetAttribute));
extern String	MonetEntity_get_attribute
  PR((MonetEntity, MonetAttribute));
extern String	MonetEntity_get_attribute_full
  PR((MonetEntity, MonetAttribute));
extern MonetBool	MonetEntity_present_attribute
  PR((MonetEntity, MonetAttribute));
extern	void	MonetEntity_delete_attribute
  PR((MonetEntity, MonetAttribute));
extern String MonetAttribute2String
  PR((MonetAttribute));
extern MonetAttribute	String2MonetAttribute
  PR((String));
extern MonetBool MonetEntity_null_attribute
  PR((MonetEntity entity, MonetAttribute attribute));
extern String MonetAttribute2FullName
  PR((MonetAttribute attribute));

extern char * MonetType_name
  PR((MonetType type));

/***************************************************************************/

/***************************************************************************/

extern MonetReportFormat MonetReportFormat_create PR((unsigned argc,
						      String *argv));
extern void MonetReportFormat_free PR((MonetReportFormat format));
extern String MonetReportFormat_simple PR((MonetReportFormat, MonetEntity));
extern String MonetReportFormat_pretty PR((MonetReportFormat, MonetEntity));
extern String MonetReportFormat_header PR((MonetReportFormat));


extern MonetEntity   String2MonetClass  PR((String));

extern void MonetEntity_set_change_trigger PR((MonetStatusTrigger change));

extern MonetEntity String2MonetEntity_subclass
  PR((Entity superclass, String name));

extern MonetEntity String2MonetEntity_instance
  PR((Entity superclass, String name));

/***************************************************************************/
/*  Types  */
/***************************************************************************/

extern MonetType String2MonetType PR((String));
extern MonetType MonetType_create
  PR((String, Boolean, AttributeRoutine, caddr_t, String));
extern void MonetType_free PR((void));
extern void MonetType_report PR((void));
extern caddr_t MonetType_find_data PR((String, AttributeRoutine));
extern void MonetAttribute_report PR((void));
extern MonetAttribute MonetAttribute_create
  PR((String name, unsigned number, MonetType, String help));

/***************************************************************************/
/***************************************************************************/
/*  $Header: Entity.h,v 1.1 91/10/01 16:06:58 a38 Exp $ */
/***************************************************************************/
/***************************************************************************/

#endif

