#ifndef MONET_SCRIPT_H
#define MONET_SCRIPT_H

typedef struct monet_member  * MonetMember;
typedef struct monet_context * MonetContext;
typedef struct monet_library * MonetLibrary;
typedef enum {
  monetapp,
  monetlib,
  monetmsh,
}  MonetLibType;
 
#include <monet/msh/Entity.h>
#include <monet/msh/Event.h>

/***********************************************************
  C Interface
  ***********************************************************/

typedef String (* ScriptCallback)
     PR((int, char**, MonetBool *));

#define MAXSCRIPTCOMMANDLEN 4096

#define MONETCALLBACK(name) \
/*ARGSUSED*/  \
  static String name (argc, argv, entity) \
  String * argv; \
  int argc; \
  MonetEntity entity; \
{ String result = "";
#define ARGCSC   (argc)
#define ARGVSC(n)   (arg[(n)])
#define RESULTSC(value) (result = (value))
#define RETURNSC return result;
#define MONETEND return result; }

/***********************************************************
  Start a new library
  ***********************************************************/

extern void MonetLibrary_begin
  PR((char *, char *));

extern void MonetLibrary_done
  PR((void));
extern void MonetLibrary_print
  PR((void));

/***********************************************************/
/*  Get next member in list  */
/***********************************************************/

extern MonetMember MonetMember_next
  PR((MonetMember));

extern String MonetMember_declaration
  PR((MonetMember));

extern String MonetMember_documentation
  PR((MonetMember));

extern String MonetMember_definition
  PR((MonetMember));

/***********************************************************/
/*  Gives help on a context  */
/***********************************************************/

extern int MonetContext_help PR((char * pattern, char * class_pattern,
				 MonetEntity stack));

/***************************************************************************/
/*  When was the command started */
/***************************************************************************/

extern long MonetContext_get_started
  PR((MonetContext context));

/***********************************************************/
/*  Current context  */
/***********************************************************/

extern MonetContext MonetContext_current
  PR((void));

/***********************************************************/
/*  Set tracing on a member and return true if was being    */
/*  traced already.  */
/***********************************************************/

extern MonetBool MonetMember_trace PR((MonetMember, Boolean));

/***********************************************************/
/*  Create command line options  */
/***********************************************************/

extern void MonetConfig_create PR((String name, String function,
				   String default_args, String help));

/***********************************************************/
/*  Create/free modules  */
/***********************************************************/

extern MonetEntity	MonetMember_module_class	PR((String name));
extern void	MonetMember_deeper	PR((void));
extern void	MonetMember_shallower	PR((void));

/***********************************************************/
/* True if the previous statement/function succeeded */
/***********************************************************/

extern MonetBool MonetPreviousResult
  PR((void));

/***********************************************************/
/*  true if must break execution  */
/***********************************************************/

extern void MonetException_raise
  PR((int code));
extern void MonetException_raise_message
  PR((int code, char * message));
extern void MonetException_register
  PR((int code, char * name));
extern void MonetException_deregister
  PR((int code));
extern char * MonetException_name
  PR((void));
extern int  MonetException_code
  PR((void));

#define MONETSH_OKAY      0
#define MONETSH_HALT      1
#define MONETSH_SIGNAL    2
#define MONETSH_RETURN    3
#define MONETSH_BREAK     4
#define MONETSH_CONTINUE  5
#define MONETSH_ERROR     6
#define MONETSH_EXIT      7
#define MONETSH_MIN_EXCEPTION 8
#define MONETSH_MAX_EXCEPTION 32

/***********************************************************/
/*  If registered this function is called by  */
/*  MonetException_code  */
/***********************************************************/

typedef void (* MonetExceptionFailer)
     PR((void));
extern void MonetException_register_fail
  PR((MonetExceptionFailer fail));

extern MonetContext MonetContext_create
  PR((Boolean interactive, char ** args, MonetBool inherit));
extern MonetContext MonetContext_free
  PR((MonetContext));
extern void  MonetContext_use
  PR((MonetEntity));
extern void  MonetContext_unuse
  PR((MonetEntity));

/***********************************************************/
/* tests for true and false */
/***********************************************************/

extern MonetBool Script_true PR((String));
#define Script_false(c) (! Script_true(c))

/***********************************************************/
/* Source prgram form file  */
/***********************************************************/

extern MonetBool  Script_source_file
  PR((String filename, String extension, String warning));

/***********************************************************/
/*  Raise exception */
/***********************************************************/

extern void MonetScriptFail PR((String, ...));
extern void MonetScriptFatal PR((String, ...));

/***********************************************************/
/*  Execute parsed script  */
/***********************************************************/

extern String MonetTree_execute
  PR((MonetEntity entity, MonetTree exec, String * args));

extern String MonetTree_execute_qualified
  PR((MonetEntity entity, MonetEntity qualifier,
      MonetTree exec, String * args));

extern String MonetMember_execute
  PR((MonetEntity entity, MonetMember,
      MonetEntity arg_entity, MonetStr id, MonetTree args));

extern MonetEntity MonetMember_entity
  PR((MonetMember));
extern MonetEvent MonetMember_event
  PR((MonetMember));

/***********************************************************/
/*  Create var and functions  */
/***********************************************************/

extern void MonetMember_free
  PR((MonetMember));

extern MonetMember MonetMember_create_local
  PR((name, value));

extern MonetMember MonetMember_create_read
  PR((String name, String * address, String help));

extern MonetMember MonetMember_create_write
  PR((String name, String * address, String help));

extern MonetMember MonetMember_create_write_long
  PR((String name, long * address, String help));

extern MonetMember MonetMember_create_write_boolean
  PR((String name, MonetBool * address, String help));

extern MonetMember MonetMember_create
  PR((String name, ScriptCallback, String syntax, String help));

extern MonetMember MonetMember_create_environment PR((
	String name, ScriptEnvironmentCallback,
	String syntax, String help));

/***********************************************************/
/*  Find  */
/***********************************************************/

extern MonetMember MonetEntity_qualified_member
  PR((MonetEntity entity, MonetStr id));

extern MonetMember MonetMember_find
  PR((String name));

extern String MonetMember2String
  PR((MonetMember));

extern String MonetMember2DeclaredString
  PR((MonetMember));

extern MonetStr MonetMember2Str
  PR((MonetMember));      

extern void MonetMember_set_entity
  PR((MonetEntity));

extern MonetMember MonetEntity_find_member
  PR((MonetEntity, MonetStr name, MonetEntity *which));

/***********************************************************/
/*  Parse and execute script  */
/***********************************************************/

extern String	MonetshEntityExecute PR((MonetEntity entity,
					 String command,
					 String *args, MonetBool interactive));
extern String	MonetshExecute PR((String command,
				   String *args, MonetBool interactive));
extern String	MonetshExecute_defined
	PR((String command, String *args, String *names,
	 String *values, MonetBool interactive));
extern MonetBool MonetshExecute_fd_async	PR((int fd, String *args,
						    MonetBool interactive));


#endif

/***********************************************************/
/***********************************************************/
/* $Header$ */
/***********************************************************/
/***********************************************************/
