/***************************************************************/
/***************************************************************/
/*  $Header$  */
/***************************************************************/
/***************************************************************/

#ifndef MONET_STRING_H
#define MONET_STRING_H

/***********************************************************/
/***********************************************************/
/*  Some machines crash when NULL passed to char * routine */
/*  char * functions                                       */
/*  strcmp is not used because it breaks on some machines  */
/*  with parameters == NULL                                */
/*  Simply testing for NULLness is not sufficient because  */
/*  "" == NULL should be True                              */
/***********************************************************/
/***********************************************************/

#define EQS(a,b) (strcmp (((a) ? (a):""), ((b) ? (b):"")) == 0)
#define NEQ(a,b) (! EQS(a,b))
#define SPACEORTAB(c) ((c) == ' ' || (c) == '\t')
#define WHITESPACE(c) ((c) == ' ' || (c) == '\n' || (c) == '\t')
#define MonetStrlen(s)  (s ? strlen (s) : 0)
#define NULSTR ((char *) NULL)

/***********************************************************/
/*  Print a count with a message and add 's' to 'after' if */
/*  count is not one.  Also print One and No instaed of    */
/*  numberic.  */
/***********************************************************/

extern void MonetPrintPrettyCount
  PR((char * before, char * after, unsigned));
extern void MonetPrintErrPrettyCount
  PR((char * before, char * after, unsigned));

extern char * MonetPrettyCount
  PR((int, char *));

/******************************************************************/
/*  Time Stamps  */
/******************************************************************/

extern char * MonetTimeStamp		PR((void));
extern char * MonetPrettyTimeStamp		PR((void));

/***********************************************************/
/*  conversions  */
/***********************************************************/

#define MonetString2long(string) (string ? atol (string) : 0)
#define MonetString2int(string)  (string ? atoi (string) : 0)

extern void MonetFreeStringOfStrings
  PR((char **s, int n));

extern MonetBool MonetEqStringOfStrings
     PR((char **s1, char **s2, int n1, int n2));

extern void MonetStringToLowerCase
  PR((char *));

extern char *		Monetlong2MegaString
  PR((long));

extern char *		unsigned2string
  PR((unsigned));

extern unsigned		String2unsigned
  PR((char *));

extern char * Monetlong2String
  PR((long));
extern char * Monetint2String
  PR((int));
extern char * MonetChar2String  PR((char c));

extern MonetBool String2MonetBool   PR((char *));

extern char *  MonetBool2String   PR((Boolean));

extern char * Monet_uptime_string PR((unsigned long timeticks));

extern char * MonetString2HexString PR((char *buf, unsigned char* cp,int len));

extern char * MonetString2AsciiString PR((char *buf, unsigned char*, int len));

extern char * MonetString2PrintString PR((char *buf, unsigned char*, int len));

extern char * MonetStringSection PR((char * string, char seperator,
				  unsigned index));

extern char * PrintString2MonetString PR((char * *source, char * buffer,
					  MonetBool escape));

extern char MonetPrintChar2char PR((char c));

/***********************************************************/
/*  Match pattern, search replace  */
/***********************************************************/

extern MonetBool MonetWordInString
  PR((char * word, char * string));

extern MonetBool	MonetMatchWildcard
  PR((char * s, char * pattern));

extern char *	MonetReplaceInString
  PR((char * source, char * search, char * replacement));

extern char *	MonetReplaceInStringBuffer
  PR((char * source, char * buffer, char * search, char * replacement));

extern MonetBool MonetIsStringInString PR ((char * str, char * pat));

extern MonetBool	MonetStringPrefixEqual
  PR((char * prefix, String));

extern char * MonetString_indent
  PR((char *, char, unsigned times));
/* Indents every line in the char * by 'times' occurances of 'char' */
/* Caller must free the result  */

extern char * MonetNextOnLine
  PR((char *));

/***********************************************************/
/*  Conactination of strings  */
/***********************************************************/

extern char *	MonetJoinStrings
  PR((int argc, char ** argv, char * between));

extern char *	MonetJoinStringToStrings
  PR((char * first, char ** argv, char * between));

extern char *	MonetJoinStringsUntilNull
  PR((char ** argv, char * between));

extern char *	MonetStringCopyToEnd
  PR((char * dest, char * src));
/*
extern char MonetReallocAppend
  PR((char **begin, int *place, char *insert, int *alloced));
*/
extern char * MonetReallocAppendString
  PR((char *, char *));

extern char *	MonetStringCopyFieldToEnd
  PR((char * dest, char * src, unsigned width));

/*  Returns the count of characters in count, must be set to zero outside */

extern char *	MonetStringCopyAndCountToEnd
  PR((char * dest, char * src, unsigned *count));

extern char *	MonetStrcat
  PR((char *, char *));

extern MonetBool	MonetCopyNextWord
  PR((char * *buf, char * result, unsigned size));

/* Split a char * into words, returns a null terminated malloced array of the
   words.  Caller must free the array and the words.  */
/* extern char** MonetSplitIntoWords PR((char *)); */

/***********************************************************/
/*  Tidy up text  */
/***********************************************************/

extern MonetBool	MonetCharInString
  PR((char c, char * set));

extern char *	MonetTidyString
  PR((char * source));

extern char *	MonetStripEndWhiteSpace
  PR((char * source));

extern char * MonetShortLabel
  PR((char *));

/***********************************************************/
/*  Parse char * */
/***********************************************************/

extern MonetBool	MonetIsWord
  PR((char * source));

/* -- looks in 'place' for next non whitespace char and returns */
extern char *	MonetNextWord
  PR((char * place));

extern char * MonetEndOfWord PR((char *));

/*extern MonetBool	MonetStringsEqual  PR((char *, char *));*/

/***********************************************************/
/*  Get system error message, uses errno  */
/***********************************************************/

extern char *	MonetSystemErrorMessage  PR((void));

/****************************************************************/
/*  Time stamp  */
/****************************************************************/

extern char * MonetPrettyTimeStamp
  PR((void));

/****************************************************************
 *
 * Memory Management
 *
 * These are equivalent to the standard routines but include a
 * bit of error checking.
 ****************************************************************/

/*  This may give warnings with the malloc because the result of malloc
    is not cast into the type of 's'  */
#define MonetRealloc(s, n)   ((char *) (s) ? realloc((char *)(s), (n)) : \
			      (char *) ((s) = malloc (n)))
#define MonetMalloc(i)       ((char *) malloc ((unsigned) (i)))
#define MonetFree(i)         { if (i) { (void) free ((char *) (i)); }}
#define MonetNew(type)       ((type *) MonetMalloc((unsigned) sizeof(type)))
#define MonetNewString(str)  ((str) == NULL ? NULL : \
			      (strcpy(MonetMalloc((unsigned)strlen(str) + 1), \
				      (str))))

#endif

/***************************************************************/
/***************************************************************/
/*  $Header$  */
/***************************************************************/
/***************************************************************/
